@@ -12,7 +12,7 @@ from sql.operators import Concat
 from trytond.config import config
 from trytond.i18n import gettext
 from trytond.model import (
-    Check, DictSchemaMixin, ModelSQL, ModelView, Workflow, fields,
+    Check, DictSchemaMixin, Index, ModelSQL, ModelView, Workflow, fields,
     sequence_ordered)
 from trytond.model.exceptions import AccessError
 from trytond.modules.company import CompanyReport
@@ -71,10 +71,9 @@ class Statement(Workflow, ModelSQL, ModelView):
 
     name = fields.Char('Name', required=True)
     company = fields.Many2One(
-        'company.company', "Company", required=True, select=True,
-        states=_states)
-    journal = fields.Many2One('account.statement.journal', 'Journal',
-        required=True, select=True,
+        'company.company', "Company", required=True, states=_states)
+    journal = fields.Many2One(
+        'account.statement.journal', "Journal", required=True,
         domain=[
             ('company', '=', Eval('company', -1)),
             ],
@@ -83,7 +82,7 @@ class Statement(Workflow, ModelSQL, ModelView):
             })
     currency = fields.Function(fields.Many2One(
             'currency.currency', "Currency"), 'on_change_with_currency')
-    date = fields.Date('Date', required=True, select=True)
+    date = fields.Date("Date", required=True)
     start_balance = Monetary(
         "Start Balance", currency='currency', digits='currency',
         states=_balance_states)
@@ -116,7 +115,7 @@ class Statement(Workflow, ModelSQL, ModelView):
             ('validated', "Validated"),
             ('cancelled', "Cancelled"),
             ('posted', "Posted"),
-            ], "State", readonly=True, select=True, sort=False)
+            ], "State", readonly=True, sort=False)
     validation = fields.Function(fields.Char('Validation'),
         'on_change_with_validation')
     to_reconcile = fields.Function(
@@ -130,6 +129,17 @@ class Statement(Workflow, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Statement, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.update({
+                Index(
+                    t,
+                    (t.journal, Index.Equality()),
+                    (t.date, Index.Range(order='DESC'))),
+                Index(
+                    t,
+                    (t.state, Index.Equality()),
+                    where=t.state.in_(['draft', 'validated'])),
+                })
         cls._order[0] = ('id', 'DESC')
         cls._transitions |= set((
                 ('draft', 'validated'),
@@ -748,8 +758,11 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
             'account.invoice': [
                 ('company', '=', Eval('company', -1)),
                 If(Bool(Eval('party')),
-                    ('party', '=', Eval('party')),
-                    ()),
+                    ['OR',
+                        ('party', '=', Eval('party', -1)),
+                        ('alternative_payees', '=', Eval('party', -1)),
+                        ],
+                    []),
                 If(Bool(Eval('account')),
                     ('account', '=', Eval('account')),
                     ()),
@@ -769,6 +782,8 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
             ('statement', '=', Eval('statement', -1)),
             ('date', '=', Eval('date', None)),
             ])
+    party_required = fields.Function(
+        fields.Boolean("Party Required"), 'on_change_with_party_required')
 
     @classmethod
     def __setup__(cls):
@@ -779,6 +794,10 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
                 | Bool(Eval('origin', 0))),
             }
         cls.account.required = True
+        cls.party.states = {
+            'required': (Eval('party_required', False)
+                & (Eval('statement_state') == 'draft')),
+            }
         t = cls.__table__()
         cls._sql_constraints += [
             ('check_statement_line_amount', Check(t, t.amount != 0),
@@ -793,6 +812,8 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
 
         table_h = cls.__table_handler__(module)
         cursor = Transaction().connection.cursor()
+
+        # Migration from 6.2: replace invoice by related_to
         if table_h.column_exist('invoice'):
             cursor.execute(*table.update(
                     [table.related_to],
@@ -813,6 +834,12 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
     def invoice(self, value):
         self.related_to = value
 
+    @fields.depends('account')
+    def on_change_with_party_required(self, name=None):
+        if self.account:
+            return self.account.party_required
+        return False
+
     @staticmethod
     def default_amount():
         return Decimal(0)
@@ -841,7 +868,8 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
 
         if self.invoice:
             if self.party:
-                if self.invoice.party != self.party:
+                if (self.invoice.party != self.party
+                        or self.party not in self.invoice.alternative_payees):
                     self.invoice = None
             else:
                 self.invoice = None
@@ -889,7 +917,13 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
     def on_change_related_to(self):
         if self.invoice:
             if not self.party:
-                self.party = self.invoice.party
+                if not self.invoice.alternative_payees:
+                    self.party = self.invoice.party
+                else:
+                    try:
+                        self.party, = self.invoice.alternative_payees
+                    except ValueError:
+                        pass
             if not self.account:
                 self.account = self.invoice.account
 
@@ -971,7 +1005,7 @@ class Line(origin_mixin(_states), sequence_ordered(), ModelSQL, ModelView):
                 to_reconcile.clear()
 
             reconcile_lines = line.invoice.get_reconcile_lines_for_amount(
-                move_line.credit - move_line.debit)
+                move_line.credit - move_line.debit, party=line.party)
 
             assert move_line.account == line.invoice.account
 
@@ -1188,7 +1222,8 @@ class ImportStatementStart(ModelView):
     __name__ = 'account.statement.import.start'
     company = fields.Many2One('company.company', "Company", required=True)
     file_ = fields.Binary("File", required=True)
-    file_format = fields.Selection([(None, '')], 'File Format', required=True)
+    file_format = fields.Selection(
+        [(None, '')], "File Format", required=True, translate=False)
 
     @classmethod
     def default_file_format(cls):
